/*
 * utf8open.c
 * Public domain
 * 2020   A. Kakuto
 *
 */

/*
  used only on windows
*/
#ifdef _WIN32 /* --ak */

#include <malloc.h>
#include <sys/stat.h>
#include <wchar.h>
#include <windows.h>
#define  LUA_LIB 1
#include "luaconf.h"
/*
  Get wide string from multibyte string. (by T. Tanaka)
*/
static wchar_t *
get_wstring_from_mbstring(int cp, const char *mbstr, wchar_t *wstr)
{
  int len;

  len = MultiByteToWideChar(cp, 0, mbstr, -1, wstr, 0);
  if (len==0) {
    return NULL;
  }
  if (wstr==NULL) {
    wstr = malloc(sizeof(wchar_t)*(len+1));
  }
  len = MultiByteToWideChar(cp, 0, mbstr, -1, wstr, len+1);
  if (len==0) {
    return NULL;
  }
  return wstr;
}

LUALIB_API FILE *fu8open(const char *filename, const char *mode)
{
  wchar_t *wfilename, *wmode;
  FILE *ret;
  int  cp;
  unsigned char *fnn;
  unsigned char *p;
  size_t len = strlen(filename);

  fnn = malloc(len + 10);
  if (len > 2 && ((filename[0] == '/' && filename[1] == '/') ||
      (filename[0] == '\\' && filename[1] == '\\' &&
       filename[2] != '?'))) {
     filename += 2;
     strcpy (fnn, "\\\\?\\UNC\\");
     strcat (fnn, filename);
  } else if (len > 2 && filename[1] == ':') {
     strcpy (fnn, "\\\\?\\");
     strcat (fnn, filename);
  } else {
     strcpy (fnn, filename);
  }
  for (p = fnn; *p; p++) {
    if (*p == '/')
       *p = '\\';
  }

  cp = CP_UTF8;
  wfilename = get_wstring_from_mbstring(cp, fnn, wfilename=NULL);
  free(fnn);
  if (wfilename == NULL)
    return NULL;
  wmode = get_wstring_from_mbstring(cp, mode, wmode=NULL);
  if (wmode == NULL)
    return NULL;
  ret = _wfopen((const wchar_t *)wfilename, (const wchar_t *)wmode);
  free(wfilename);
  free(wmode);
  return ret;
}

LUALIB_API FILE *fu8reopen(const char *filename, const char *mode, FILE *ff)
{
  wchar_t *wfilename, *wmode;
  FILE *ret;
  int  cp;
  unsigned char *fnn;
  unsigned char *p;
  size_t len = strlen(filename);

  fnn = malloc(len + 10);
  if (len > 2 && ((filename[0] == '/' && filename[1] == '/') ||
      (filename[0] == '\\' && filename[1] == '\\' &&
       filename[2] != '?'))) {
     filename += 2;
     strcpy (fnn, "\\\\?\\UNC\\");
     strcat (fnn, filename);
  } else if (len > 2 && filename[1] == ':') {
     strcpy (fnn, "\\\\?\\");
     strcat (fnn, filename);
  } else {
     strcpy (fnn, filename);
  }
  for (p = fnn; *p; p++) {
    if (*p == '/')
       *p = '\\';
  }

  cp = CP_UTF8;
  wfilename = get_wstring_from_mbstring(cp, fnn, wfilename=NULL);
  free(fnn);
  if (wfilename == NULL)
    return NULL;
  wmode = get_wstring_from_mbstring(cp, mode, wmode=NULL);
  if (wmode == NULL)
    return NULL;
  ret = _wfreopen((const wchar_t *)wfilename, (const wchar_t *)wmode, ff);
  free(wfilename);
  free(wmode);
  return ret;
}

LUALIB_API int u8_stat(const char *path, struct _stat *buffer)
{
  wchar_t *wpath;
  int     ret;

  wpath = get_wstring_from_mbstring(CP_UTF8, path, wpath = NULL);
  ret = _wstat(wpath, buffer);
  free(wpath);
  return ret;
}

LUALIB_API int u8_stati64(const char *path, struct _stati64 *buffer)
{
  wchar_t *wpath;
  int     ret;

  wpath = get_wstring_from_mbstring(CP_UTF8, path, wpath = NULL);
  ret = _wstati64(wpath, buffer);
  free(wpath);
  return ret;
}
#endif /* _WIN32 --ak */
