/*
 * chgstrcp.c
 * Public domain
 * 2019   A. Kakuto
 *
 * lua module "chgstrcp"
 * chgstrcp.syscptoutf8(str)
 * return utf8 string, where str should be in system code page
 * chgstrcp.utf8tosyscp(str)
 * return string in system code page, where str should be utf8
 */

/*
  used only on windows
*/
#ifdef _WIN32 /* --ak */
#include <lua.h>
#include <lauxlib.h>
#include <kpathsea/config.h>
#include <kpathsea/c-ctype.h>
#include <kpathsea/line.h>
#include <kpathsea/readable.h>
#include <kpathsea/variable.h>
#include <kpathsea/absolute.h>
#include <kpathsea/knj.h>
#include <wchar.h>

static int syscptoutf8(lua_State * L)
{
  wchar_t *ws;
  char *s;
  const char *st = luaL_checkstring(L, 1);
  int syscp = AreFileApisANSI() ? GetACP() : GetOEMCP();
  ws = get_wstring_from_mbstring(syscp, st, ws=NULL);
  s  = get_mbstring_from_wstring(CP_UTF8, ws, s=NULL);
  free(ws);
  lua_pushstring(L, s);
  return 1;
}

static int utf8tosyscp(lua_State * L)
{
  wchar_t *ws;
  char *s;
  const char *st = luaL_checkstring(L, 1);
  int syscp = AreFileApisANSI() ? GetACP() : GetOEMCP();
  ws = get_wstring_from_mbstring(CP_UTF8, st, ws=NULL);
  s  = get_mbstring_from_wstring(syscp, ws, s=NULL);
  free(ws);
  lua_pushstring(L, s);
  return 1;
}

static const luaL_Reg chgstrcp[] = {
  {"syscptoutf8", syscptoutf8},
  {"utf8tosyscp", utf8tosyscp},
  {NULL, NULL}
};

int luaopen_chgstrcp(lua_State * L)
{
  luaL_register(L, "chgstrcp", chgstrcp);
  return 1;
}
#endif /* _WIN32 --ak */
